@@ -64,15 +64,14 @@ use tracing::{instrument, warn};
 use crate::commands::smartlog::smartlog;
 use crate::core::config::get_restack_preserve_timestamps;
 use crate::core::eventlog::{EventLogDb, EventReplayer};
-use crate::core::graph::{
-    make_graph, resolve_commits, BranchOids, HeadOid, MainBranchOid, ResolveCommitsResult,
-};
-use crate::core::mergebase::make_merge_base_db;
 use crate::core::rewrite::{
     execute_rebase_plan, find_abandoned_children, find_rewrite_target, move_branches,
     BuildRebasePlanOptions, ExecuteRebasePlanOptions, RebasePlanBuilder,
 };
-use crate::git::{GitRunInfo, NonZeroOid, Repo};
+use crate::git::{
+    resolve_commits, sort_commit_set, CommitSet, Dag, GitRunInfo, NonZeroOid, Repo,
+    ResolveCommitsResult,
+};
 use crate::tui::Effects;
 
 #[instrument(skip(commits))]
@@ -86,51 +85,48 @@ fn restack_commits(
     build_options: &BuildRebasePlanOptions,
     execute_options: &ExecuteRebasePlanOptions,
 ) -> eyre::Result<isize> {
+    let references_snapshot = repo.get_references_snapshot()?;
     let event_replayer = EventReplayer::from_event_log_db(effects, repo, event_log_db)?;
     let event_cursor = event_replayer.make_default_cursor();
-    let merge_base_db = make_merge_base_db(effects, repo, conn, &event_replayer)?;
-    let head_oid = repo.get_head_info()?.oid;
-    let main_branch_oid = repo.get_main_branch_oid()?;
-    let branch_oid_to_names = repo.get_branch_oid_to_names()?;
-    let graph = make_graph(
+    let dag = Dag::open_and_sync(
         effects,
         repo,
-        &merge_base_db,
         &event_replayer,
         event_cursor,
-        &HeadOid(head_oid),
-        &MainBranchOid(main_branch_oid),
-        &BranchOids(branch_oid_to_names.keys().copied().collect()),
-        true,
+        &references_snapshot,
     )?;
 
+    let commit_set: CommitSet = match commits {
+        Some(commits) => commits.into_iter().collect(),
+        None => dag.obsolete_commits.clone(),
+    };
+    let commits = sort_commit_set(repo, &dag, &commit_set)?;
+
     struct RebaseInfo {
         dest_oid: NonZeroOid,
         abandoned_child_oids: Vec<NonZeroOid>,
     }
-    let commits: HashSet<NonZeroOid> = match commits {
-        Some(commits) => commits.into_iter().collect(),
-        None => graph.keys().copied().collect(),
-    };
-    let rebases: Vec<RebaseInfo> = commits
-        .into_iter()
-        .filter_map(|original_oid| {
-            find_abandoned_children(&graph, &event_replayer, event_cursor, original_oid).map(
-                |(rewritten_oid, abandoned_child_oids)| RebaseInfo {
+    let rebases: Vec<RebaseInfo> = {
+        let mut result = Vec::new();
+        for original_commit in commits {
+            let abandoned_children = find_abandoned_children(
+                &dag,
+                &event_replayer,
+                event_cursor,
+                original_commit.get_oid(),
+            )?;
+            if let Some((rewritten_oid, abandoned_child_oids)) = abandoned_children {
+                result.push(RebaseInfo {
                     dest_oid: rewritten_oid,
                     abandoned_child_oids,
-                },
-            )
-        })
-        .collect();
+                });
+            }
+        }
+        result
+    };
 
     let rebase_plan = {
-        let mut builder = RebasePlanBuilder::new(
-            repo,
-            &graph,
-            &merge_base_db,
-            &MainBranchOid(main_branch_oid),
-        );
+        let mut builder = RebasePlanBuilder::new(repo, &dag, references_snapshot.main_branch_oid);
         for RebaseInfo {
             dest_oid,
             abandoned_child_oids,
@@ -189,21 +185,6 @@ fn restack_branches(
     options: &ExecuteRebasePlanOptions,
 ) -> eyre::Result<isize> {
     let event_replayer = EventReplayer::from_event_log_db(effects, repo, event_log_db)?;
-    let merge_base_db = make_merge_base_db(effects, repo, conn, &event_replayer)?;
-    let head_oid = repo.get_head_info()?.oid;
-    let main_branch_oid = repo.get_main_branch_oid()?;
-    let branch_oid_to_names = repo.get_branch_oid_to_names()?;
-    let graph = make_graph(
-        effects,
-        repo,
-        &merge_base_db,
-        &event_replayer,
-        event_replayer.make_default_cursor(),
-        &HeadOid(head_oid),
-        &MainBranchOid(main_branch_oid),
-        &BranchOids(branch_oid_to_names.keys().copied().collect()),
-        true,
-    )?;
 
     let mut rewritten_oids = HashMap::new();
     for branch in repo.get_all_local_branches()? {
@@ -217,12 +198,8 @@ fn restack_branches(
                 continue;
             }
         };
-        if !graph.contains_key(&branch_target) {
-            continue;
-        }
 
         if let Some(new_oid) = find_rewrite_target(
-            &graph,
             &event_replayer,
             event_replayer.make_default_cursor(),
             branch_target,
@@ -330,6 +307,6 @@ pub fn restack(
         None => result,
     };
 
-    smartlog(effects)?;
+    smartlog(effects, &Default::default())?;
     Ok(result)
 }
