@@ -3,8 +3,5 @@
 pub mod config;
 pub mod eventlog;
 pub mod formatting;
-pub mod graph;
-pub mod mergebase;
 pub mod metadata;
 pub mod rewrite;
-pub mod tui;
