@@ -5,14 +5,13 @@ use std::ops::Sub;
 use std::path::PathBuf;
 
 use chashmap::CHashMap;
+use eden_dag::DagAlgorithm;
 use itertools::Itertools;
 use rayon::{prelude::*, ThreadPool, ThreadPoolBuilder};
 use tracing::{instrument, warn};
 
 use crate::core::formatting::printable_styled_string;
-use crate::core::graph::{CommitGraph, MainBranchOid};
-use crate::core::mergebase::MergeBaseDb;
-use crate::git::{Commit, NonZeroOid, PatchId, Repo};
+use crate::git::{commit_set_to_vec, Commit, CommitSet, Dag, NonZeroOid, PatchId, Repo};
 use crate::tui::{Effects, OperationType};
 
 thread_local! {
@@ -151,11 +150,10 @@ struct BuildState {
 
 /// Builder for a rebase plan. Unlike regular Git rebases, a `git-branchless`
 /// rebase plan can move multiple unrelated subtrees to unrelated destinations.
-#[derive(Debug)]
-pub struct RebasePlanBuilder<'repo, M: MergeBaseDb + 'repo> {
+#[derive(Clone, Debug)]
+pub struct RebasePlanBuilder<'repo> {
     repo: &'repo Repo,
-    graph: &'repo CommitGraph<'repo>,
-    merge_base_db: &'repo M,
+    dag: &'repo Dag,
     main_branch_oid: NonZeroOid,
 
     /// There is a mapping from from `x` to `y` if `x` must be applied before
@@ -163,21 +161,6 @@ pub struct RebasePlanBuilder<'repo, M: MergeBaseDb + 'repo> {
     initial_constraints: HashMap<NonZeroOid, HashSet<NonZeroOid>>,
 }
 
-/// Can't `#[derive(Clone)]` because of the parametrized `M`, which isn't
-/// clonable. (Even though we're just storing a reference to `M`, which is
-/// clonable, the `derive` macro doesn't know that.)
-impl<'repo, M: MergeBaseDb + 'repo> Clone for RebasePlanBuilder<'repo, M> {
-    fn clone(&self) -> Self {
-        Self {
-            repo: self.repo,
-            graph: self.graph,
-            merge_base_db: self.merge_base_db,
-            main_branch_oid: self.main_branch_oid,
-            initial_constraints: self.initial_constraints.clone(),
-        }
-    }
-}
-
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 struct Constraint {
     parent_oid: NonZeroOid,
@@ -255,20 +238,13 @@ impl BuildRebasePlanError {
     }
 }
 
-impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
+impl<'repo> RebasePlanBuilder<'repo> {
     /// Constructor.
-    pub fn new(
-        repo: &'repo Repo,
-        graph: &'repo CommitGraph,
-        merge_base_db: &'repo M,
-        main_branch_oid: &MainBranchOid,
-    ) -> Self {
-        let MainBranchOid(main_branch_oid) = main_branch_oid;
+    pub fn new(repo: &'repo Repo, dag: &'repo Dag, main_branch_oid: NonZeroOid) -> Self {
         RebasePlanBuilder {
             repo,
-            graph,
-            merge_base_db,
-            main_branch_oid: *main_branch_oid,
+            dag,
+            main_branch_oid,
             initial_constraints: Default::default(),
         }
     }
@@ -471,62 +447,22 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
     #[instrument]
     fn collect_descendants(
         &self,
-        effects: &Effects,
+        visible_commits: &CommitSet,
         acc: &mut Vec<Constraint>,
         current_oid: NonZeroOid,
     ) -> eyre::Result<()> {
-        // FIXME: O(n^2) algorithm.
-        for (child_oid, node) in self.graph.iter() {
-            if node.commit.get_parent_oids().contains(&current_oid) {
-                acc.push(Constraint {
-                    parent_oid: current_oid,
-                    child_oid: *child_oid,
-                });
-                self.collect_descendants(effects, acc, *child_oid)?;
-            }
-        }
-
-        // Calculate the commits along the main branch to be moved if this is a
-        // constraint for a main branch commit.
-        let is_main = match self.graph.get(&current_oid) {
-            Some(node) => node.is_main,
-            None => true,
-        };
-        if is_main {
-            // This must be a main branch commit. We need to collect its
-            // descendants, which don't appear in the commit graph.
-            let path = self.merge_base_db.find_path_to_merge_base(
-                effects,
-                self.repo,
-                self.main_branch_oid,
-                current_oid,
-            )?;
-            if let Some(path) = path {
-                let mut parent_oid = current_oid;
-                for child_commit in path
-                    .into_iter()
-                    // Start from the node and traverse children towards the main branch.
-                    .rev()
-                    // Skip the starting node itself, as it already has a constraint.
-                    .skip(1)
-                {
-                    let child_oid = child_commit.get_oid();
-                    acc.push(Constraint {
-                        parent_oid,
-                        child_oid,
-                    });
-                    // We've hit a node that is in the graph, so further
-                    // constraints should be added by the above code path.
-                    //
-                    // FIXME: this may be incorrect for multi-parent commits,
-                    // since `find_path_to_merge_base` actually returns a
-                    // partially-ordered set of commits.
-                    if self.graph.contains_key(&child_oid) {
-                        break;
-                    }
-                    parent_oid = child_oid;
-                }
-            }
+        let children_oids = self
+            .dag
+            .query()
+            .children(CommitSet::from(current_oid))?
+            .intersection(visible_commits);
+        let children_oids = commit_set_to_vec(&children_oids)?;
+        for child_oid in children_oids {
+            acc.push(Constraint {
+                parent_oid: current_oid,
+                child_oid,
+            });
+            self.collect_descendants(visible_commits, acc, child_oid)?;
         }
         Ok(())
     }
@@ -535,15 +471,21 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
     /// of a referred-to commit. This adds enough information to the constraint
     /// graph that it now represents the actual end-state commit graph that we
     /// want to create, not just a list of constraints.
-    fn add_descendant_constraints(
-        &self,
-        effects: &Effects,
-        state: &mut BuildState,
-    ) -> eyre::Result<()> {
+    fn add_descendant_constraints(&self, state: &mut BuildState) -> eyre::Result<()> {
         let all_descendants_of_constrained_nodes = {
+            let public_commits = self.dag.query_public_commits()?;
+            let active_heads = self.dag.query_active_heads(
+                &public_commits,
+                &self
+                    .dag
+                    .observed_commits
+                    .difference(&self.dag.obsolete_commits),
+            )?;
+            let visible_commits = self.dag.query().ancestors(active_heads)?;
+
             let mut acc = Vec::new();
             for parent_oid in state.constraints.values().flatten().cloned() {
-                self.collect_descendants(effects, &mut acc, parent_oid)?;
+                self.collect_descendants(&visible_commits, &mut acc, parent_oid)?;
             }
             acc
         };
@@ -663,7 +605,7 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
                 Self::get_constraints_sorted_for_debug(&state),
             );
         }
-        self.add_descendant_constraints(&effects, &mut state)?;
+        self.add_descendant_constraints(&mut state)?;
         if *dump_rebase_constraints {
             // For test: don't print to `effects.get_output_stream()`, as it will
             // be suppressed.
@@ -757,6 +699,9 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
         }
     }
 
+    /// Get the patch IDs for commits between `current_oid` and `dest_oid`,
+    /// filtered to only the commits which might have the same patch ID as a
+    /// commit being rebased.
     #[instrument]
     fn get_upstream_patch_ids(
         &self,
@@ -766,19 +711,16 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
         dest_oid: NonZeroOid,
     ) -> eyre::Result<HashSet<PatchId>> {
         let merge_base_oid =
-            self.merge_base_db
-                .get_merge_base_oid(effects, self.repo, dest_oid, current_oid)?;
+            self.dag
+                .get_one_merge_base_oid(effects, self.repo, dest_oid, current_oid)?;
         let merge_base_oid = match merge_base_oid {
             None => return Ok(HashSet::new()),
             Some(merge_base_oid) => merge_base_oid,
         };
 
-        let path = self.merge_base_db.find_path_to_merge_base(
-            effects,
-            self.repo,
-            dest_oid,
-            merge_base_oid,
-        )?;
+        let path =
+            self.dag
+                .find_path_to_merge_base(effects, self.repo, dest_oid, merge_base_oid)?;
         let path = match path {
             None => return Ok(HashSet::new()),
             Some(path) => path,
@@ -794,6 +736,7 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
                 .map(|oid| self.repo.find_commit(*oid))
                 .flatten_ok()
                 .try_collect()?;
+
             self.filter_path_to_merge_base_commits(effects, state, &pool, path, touched_commits)?
         };
 
@@ -858,11 +801,10 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
     ) -> eyre::Result<Vec<Commit<'repo>>> {
         let (effects, _progress) = effects.start_operation(OperationType::FilterCommits);
 
-        let local_touched_paths: HashSet<PathBuf> = touched_commits
+        let local_touched_paths: Vec<HashSet<PathBuf>> = touched_commits
             .into_iter()
             .map(|commit| self.repo.get_paths_touched_by_commit(&commit))
             .filter_map(|x| x.transpose())
-            .flatten_ok()
             .try_collect()?;
 
         let filtered_path = {
@@ -878,7 +820,7 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
                 path.into_par_iter()
                     .map(|commit_oid| {
                         if let Some(upstream_touched_paths) = touched_paths_cache.get(&commit_oid) {
-                            if Self::is_candidate_to_check_patch_id(
+                            if Self::should_check_patch_id(
                                 &*upstream_touched_paths,
                                 &local_touched_paths,
                             ) {
@@ -898,7 +840,7 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
                             };
                             let upstream_touched_paths =
                                 repo.get_paths_touched_by_commit(&commit)?;
-                            let result = if Self::is_candidate_to_check_patch_id(
+                            let result = if Self::should_check_patch_id(
                                 &upstream_touched_paths,
                                 &local_touched_paths,
                             ) {
@@ -923,17 +865,18 @@ impl<'repo, M: MergeBaseDb + 'repo> RebasePlanBuilder<'repo, M> {
         Ok(filtered_path)
     }
 
-    fn is_candidate_to_check_patch_id(
+    fn should_check_patch_id(
         upstream_touched_paths: &Option<HashSet<PathBuf>>,
-        local_touched_paths: &HashSet<PathBuf>,
+        local_touched_paths: &[HashSet<PathBuf>],
     ) -> bool {
         match upstream_touched_paths {
             Some(upstream_touched_paths) => {
-                // This could be more specific -- we could check to see if they are
-                // exactly the same. I'm checking only if there is some intersection to
-                // make sure there's not some edge-case e.g. around renames that I've
-                // missed.
-                !local_touched_paths.is_disjoint(upstream_touched_paths)
+                // It's possible that the same commit was applied after a parent
+                // commit renamed a certain path. In that case, this check won't
+                // trigger. We'll rely on the empty-commit check after the
+                // commit has been made to deduplicate the commit in that case.
+                // FIXME: this code path could be optimized further.
+                local_touched_paths.iter().contains(upstream_touched_paths)
             }
             None => true,
         }
