@@ -1,7 +1,7 @@
 use branchless::testing::make_git;
 
 #[test]
-fn test_gc() -> anyhow::Result<()> {
+fn test_gc() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
@@ -11,7 +11,7 @@ fn test_gc() -> anyhow::Result<()> {
 
     {
         let repo = git.get_repo()?;
-        assert!(matches!(repo.revparse_single("62fc20d2"), Ok(_)));
+        assert!(matches!(repo.revparse_single_commit("62fc20d2"), Ok(_)));
     }
 
     git.run(&["gc", "--prune=now"])?;
@@ -42,7 +42,7 @@ branchless: collecting garbage
 
     {
         let repo = git.get_repo()?;
-        assert!(matches!(repo.revparse_single("62fc20d2"), Err(_)));
+        assert!(repo.revparse_single_commit("62fc20d2")?.is_none())
     }
 
     Ok(())
